implementation class
NonTerminal: Symbol
{
  <doc> The number of anonymous non-terminals.  </doc>
  static int num_anonymous;
}

<doc> Return a newly created anonymous non-terminal, which has been added
    to the {parser}.  </doc>
instance (id) (result)
  newAnonymous
{
  String s = [[MutableByteString new] print ("T", num_anonymous++)];
  result = [[self alloc] init s];
  [parser addNonTerminal result];
}

end;

implementation instance
NonTerminal
{
  <doc> Our name.  </doc>
  public String name;

  <doc> Flat rules of this non-terminal.  </doc>
  public MutableArray flat_rules;

  <doc> Iff {TRUE}, this non-terminal has been used in the rhs of some rule.
      </doc>
  public boolean used;
}

<doc> Designated initializer </doc>
id
    init String n
{
  name = n;
  = [super init];
}

void
  add-flat-rule FlatRule r
{
  if (!flat_rules)
    flat_rules = [MutableObjectArray new];
  [flat_rules add r];
}

void
  describe-flat-rules
{
  int i, n = [flat_rules length];
  for (i = 0; i < n; i++)
    [flat_rules[i] describe-flat-rule [stdio err]];
}

OutputStream (s)
  describe-flat-rule OutputStream s
{
  [s print name];
}

void
  checkLL1
{
  [[GPParse ([GPParse current_parser]) rules][name] checkLL1];
}

OutputStream
	 compile OutputStream s
  withAttributes Array actual
{
  Array formal = [[GPParse ([GPParse current_parser]) rules][name] attributes];
  int n_attr = !actual ? 0 : [actual length];

  if (formal != nil && [formal length] != n_attr
      || !formal && n_attr != 0)
    [[[[GPParse current_parser] error self] print ("incorrect attributes to `", name, '\'')] nl];
  else
    {
      int n_out_attr, i;
      Attribute a;

      for (i = 0; i < n_attr; i++)
	{
	  a = formal[i];

	  if (![a in_not_out])
	    {
	      if (!n_out_attr++)
		[s print (indent, "(")];
	      else
		[s print ", "];
	      [s print ({Any a1 = actual[i];})];
	    }
	}
      if (!n_out_attr)
	[s print (indent, "[self ", name)];
      else
	[s print (") = [self ", name)];

      if (n_attr != n_out_attr)
	{
	  boolean first = YES;

	  for (i = 0; i < n_attr; i++)
	    {
	      a = formal[i];

	      if ([a in_not_out])
		{
		  // CCC Likewise...
		  // Sat Jun 22 23:10:17 1996, tiggr@tricky.es.ele.tue.nl
		  [s print (String (first ? " (" : ", "), Any (actual[i]))];
		  first = NO;
		}
	    }
	  [s print ')'];
	}

      [s print "];"];
    }

  = s;
}

IntegerRangeSet
  computeLA IntegerRangeSet f
{
  Rule r = [GPParse ([GPParse current_parser]) rules][name];

  follow = f;
  la = [r computeLA f];
  first = [r first];

  = la;
}

<doc> Note that we are {used}, before passing to {super}.  </doc>
id
  makeFlatRules String rule_name
{
  used = TRUE;
  = [super makeFlatRules rule_name];
}

OutputStream (s)
  writeFields OutputStream s
{
  = [[super writeFields s] print (" name=", name)];
}

end;
